"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gram_ai import models, utils
from gram_ai._hooks import HookContext
from gram_ai.types import OptionalNullable, UNSET
from typing import Any, Mapping, Optional, Union


class Templates(BaseSDK):
    r"""Manages re-usable prompt templates and higher-order tools for a project."""

    def get(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        gram_key: Optional[str] = None,
        gram_session: Optional[str] = None,
        gram_project: Optional[str] = None,
        security: Optional[
            Union[models.GetTemplateSecurity, models.GetTemplateSecurityTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetPromptTemplateResult:
        r"""getTemplate templates

        Get prompt template by its ID or name.

        :param id: The ID of the prompt template
        :param name: The name of the prompt template
        :param gram_key: API Key header
        :param gram_session: Session header
        :param gram_project: project header
        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTemplateRequest(
            id=id,
            name=name,
            gram_key=gram_key,
            gram_session=gram_session,
            gram_project=gram_project,
        )

        req = self._build_request(
            method="GET",
            path="/rpc/templates.get",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, Optional[models.GetTemplateSecurity]
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTemplate",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "415",
                "422",
                "4XX",
                "500",
                "502",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetPromptTemplateResult)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "409", "415", "422"],
            "application/json",
        ):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, ["500", "502"], "application/json"):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        gram_key: Optional[str] = None,
        gram_session: Optional[str] = None,
        gram_project: Optional[str] = None,
        security: Optional[
            Union[models.GetTemplateSecurity, models.GetTemplateSecurityTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetPromptTemplateResult:
        r"""getTemplate templates

        Get prompt template by its ID or name.

        :param id: The ID of the prompt template
        :param name: The name of the prompt template
        :param gram_key: API Key header
        :param gram_session: Session header
        :param gram_project: project header
        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTemplateRequest(
            id=id,
            name=name,
            gram_key=gram_key,
            gram_session=gram_session,
            gram_project=gram_project,
        )

        req = self._build_request_async(
            method="GET",
            path="/rpc/templates.get",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, Optional[models.GetTemplateSecurity]
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTemplate",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "415",
                "422",
                "4XX",
                "500",
                "502",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetPromptTemplateResult)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "409", "415", "422"],
            "application/json",
        ):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, ["500", "502"], "application/json"):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def render(
        self,
        *,
        id: str,
        render_template_request_body: Union[
            models.RenderTemplateRequestBody, models.RenderTemplateRequestBodyTypedDict
        ],
        gram_key: Optional[str] = None,
        gram_session: Optional[str] = None,
        gram_project: Optional[str] = None,
        security: Optional[
            Union[models.RenderTemplateSecurity, models.RenderTemplateSecurityTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RenderTemplateResult:
        r"""renderTemplate templates

        Render a prompt template given some input data.

        :param id: The ID of the prompt template to render
        :param render_template_request_body:
        :param gram_key: API Key header
        :param gram_session: Session header
        :param gram_project: project header
        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RenderTemplateRequest(
            id=id,
            gram_key=gram_key,
            gram_session=gram_session,
            gram_project=gram_project,
            render_template_request_body=utils.get_pydantic_model(
                render_template_request_body, models.RenderTemplateRequestBody
            ),
        )

        req = self._build_request(
            method="POST",
            path="/rpc/templates.render",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, Optional[models.RenderTemplateSecurity]
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.render_template_request_body,
                False,
                False,
                "json",
                models.RenderTemplateRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="renderTemplate",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "415",
                "422",
                "4XX",
                "500",
                "502",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.RenderTemplateResult)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "409", "415", "422"],
            "application/json",
        ):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, ["500", "502"], "application/json"):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def render_async(
        self,
        *,
        id: str,
        render_template_request_body: Union[
            models.RenderTemplateRequestBody, models.RenderTemplateRequestBodyTypedDict
        ],
        gram_key: Optional[str] = None,
        gram_session: Optional[str] = None,
        gram_project: Optional[str] = None,
        security: Optional[
            Union[models.RenderTemplateSecurity, models.RenderTemplateSecurityTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RenderTemplateResult:
        r"""renderTemplate templates

        Render a prompt template given some input data.

        :param id: The ID of the prompt template to render
        :param render_template_request_body:
        :param gram_key: API Key header
        :param gram_session: Session header
        :param gram_project: project header
        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RenderTemplateRequest(
            id=id,
            gram_key=gram_key,
            gram_session=gram_session,
            gram_project=gram_project,
            render_template_request_body=utils.get_pydantic_model(
                render_template_request_body, models.RenderTemplateRequestBody
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/rpc/templates.render",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, Optional[models.RenderTemplateSecurity]
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.render_template_request_body,
                False,
                False,
                "json",
                models.RenderTemplateRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="renderTemplate",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "415",
                "422",
                "4XX",
                "500",
                "502",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.RenderTemplateResult)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "409", "415", "422"],
            "application/json",
        ):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, ["500", "502"], "application/json"):
            response_data = utils.unmarshal_json(http_res.text, models.ServiceErrorData)
            raise models.ServiceError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )
